
# q-expansion of newform 5776.2.a.be, downloaded from the LMFDB on 16 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5776
weight = 2
poly_data = [-1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 0], [0, 2], [-3, 1], [2, -2], [2, -2], [-2, -1], [-4, -1], [0, 0], [-4, 2], [6, -5], [0, 2], [-9, -1], [1, 3], [8, -2], [-2, 4], [3, -5], [2, 4], [2, 1], [8, -6], [-2, -8], [-5, 1], [-4, -2], [-2, -4], [-7, 3], [-8, 1], [-15, 1], [6, -14], [2, -12], [3, -13], [8, -3], [-4, 12], [-2, 4], [3, 1], [14, 0], [-10, 5], [12, -4], [10, 1], [-2, 14], [-10, 4], [3, 7], [-16, 12], [2, 12], [-6, -4], [14, 4], [-6, -3], [-8, 4], [-6, 2], [24, -6], [-4, 8], [5, 1], [-4, -9], [-24, 10], [18, -12], [6, 6], [-19, -1], [2, 8], [-10, 7], [-26, 0], [-7, 5], [-8, 21], [-20, 4], [-18, 8], [8, -6], [-30, 2], [-5, 17], [3, 7], [22, -6], [-3, -1], [-6, -14], [-30, 5], [-12, 19], [-14, -4], [-4, 24], [-14, 7], [-28, 12], [22, 2], [6, -11], [-10, 20], [-14, 8], [-7, 15], [4, -4], [-17, -5], [-2, -14], [17, -13], [28, -2], [-8, 2], [16, -11], [11, -23], [14, -12], [10, 8], [16, -28], [-4, 6], [18, -2], [4, 4], [18, -28], [-8, -2], [-26, 27], [-4, -15], [-4, 0], [-26, 1], [10, 8], [10, 8], [30, 6], [-11, -5], [14, -30], [28, -29], [-14, -18], [-17, 29], [-28, 18], [-14, 4], [-28, 32], [9, 1], [38, -8], [-10, 20], [20, -18], [4, -15], [24, -24], [-10, 26], [-15, 33], [-30, 30], [14, -12], [-31, -1], [6, -1], [-14, 10], [10, -20], [17, -35], [10, 5], [-16, -6], [26, -8], [2, 5], [2, 18], [16, 16], [-8, 26], [10, 9], [14, -20], [27, -19], [10, 3], [26, -4], [-34, -9], [-9, -19], [-8, -22], [10, -15], [6, -24], [-12, 30], [19, 15], [22, 0], [-26, 5], [0, 9], [-12, 26], [-16, 16], [10, 19], [-9, -23], [24, 10], [2, -8], [-4, -6], [-4, -26], [-36, 16], [-40, 31], [6, -16], [-26, 0], [-2, 10], [-20, 45], [6, 24], [42, -30], [1, -13], [6, -22], [-22, 14], [-3, 17], [24, -23], [38, -39], [-8, -6], [10, 20], [26, -22], [-29, 1], [8, 8], [24, -13], [-34, 42], [2, 21], [26, -20], [-34, -11], [40, 4], [8, -32], [-30, 19], [-11, -29], [-40, -2], [-3, -11], [-1, -1], [-34, 44], [24, -7], [8, -30], [-27, 27], [-2, 16], [-16, 14], [-29, 31], [16, -48], [10, -24], [24, -21], [5, 33], [-44, 39], [-8, 10], [-50, -5], [-28, -18], [-2, -1], [8, 5], [10, 12], [-23, 25], [-8, 2], [16, -12], [-17, -27], [4, -22], [-13, -11], [1, -35], [28, 0], [66, 2], [-40, 10], [16, -15], [-6, -14], [-32, 9], [2, 22], [53, 5], [26, -16], [-26, 60], [32, -55], [34, -16], [32, 20], [-43, 25], [28, -43], [-22, 50], [-38, 6], [-10, 8], [-50, 8], [-46, 30], [-18, -4], [-30, 0], [0, 38], [-18, -26], [4, -9], [14, -24], [16, 2], [-24, -6], [24, -2], [32, -2], [-10, -28], [11, 19], [-48, 29], [22, 6], [-16, 24], [32, 16], [-44, 10], [-34, -8], [13, -7], [7, -47], [-14, 2], [4, 33], [-18, -28], [-36, 2], [17, 25], [28, -32], [26, 12], [6, 24], [-28, -10], [18, -46], [6, 12], [-30, -16], [-76, -1], [26, -8], [22, 16], [-4, 19], [46, -58], [2, 8], [-29, 3], [28, -22], [28, -41], [2, -44], [9, -41], [12, -2], [26, -34], [-5, -21], [-36, 57], [32, -44], [4, -40], [-28, 20], [-38, -8], [18, -5], [-20, -32], [-26, -26], [19, 9], [-2, 11], [-2, -28], [50, -44], [18, -59], [40, 18], [1, 31], [2, 6], [-2, 31], [-7, -15], [16, -20], [-42, 16], [38, -40], [-40, -28], [-29, 21], [-26, 59], [-32, 64], [8, -8], [-40, 48], [-36, 45], [36, 0], [-53, 35], [2, -36], [41, -27], [-30, 44], [38, -16], [-15, -5], [-10, 12], [62, 14], [-44, 45], [30, 8], [42, 20], [-48, -13], [43, 25], [12, 12], [32, 7], [23, -13], [42, -14], [-25, -3], [-20, 21], [34, -22], [42, 0], [-46, 30], [10, -39], [49, 17], [10, 28], [-68, 8], [-4, -42], [8, -44], [-40, -14], [-14, -1], [-7, -49], [12, 43], [-70, 6], [18, -68], [-38, 12], [26, -12], [16, -42], [78, 3], [46, -24], [77, 5], [20, -44], [30, 8], [-54, 9], [-62, 4], [-19, -21], [-66, 13], [8, 20], [-38, -5], [49, 23], [-56, 40], [20, 14], [39, -59], [48, -24], [50, 21], [28, -21], [26, -28], [28, -36], [-54, 40], [44, -31], [14, 8], [-6, 10], [60, -51], [54, -8], [-12, 22], [20, 8], [-14, -44], [-50, 28], [-34, 21], [38, 4], [26, 34], [24, 3], [31, -43], [-72, 21], [-42, -5], [23, -51], [0, 20], [-49, 41], [4, 22], [26, -36], [-32, 70], [14, 29], [10, -32], [-42, 2], [0, 29], [55, -69], [12, -46], [-38, 6], [-72, -4], [-40, -3], [-18, 14], [-90, 4], [16, -46], [2, 17], [-33, -17], [-1, -31], [34, -26], [-20, 29], [-29, 11], [-60, 8], [-86, -7], [20, -3], [-32, 40], [-62, 32], [-48, -11], [10, 13], [-8, -38], [60, -4], [67, -15], [-62, 56], [-54, -30], [62, 0], [-37, 41], [0, 16], [-58, 27], [11, -33], [66, -32], [-6, -16], [37, 27], [31, -21], [-38, -22], [3, -55], [46, 34], [-18, -32]]
